﻿/////////////////////////////////////////////////////////////////////////////////
//
// File: websch.js
// Defines: $.addGoogleSearch
// Dependencies: jquery
// Description: defines the function for adding google search
//
/////////////////////////////////////////////////////////////////////////////////
/// <reference path="http://code.jquery.com/jquery.min.js"/>

(function ($) {
    // defaults object. we'll stick this object into the
    var defaults =
    {
        isASOn: true
    };

    /// <summary>
    ///     add google search for each jquery objects
    /// </summary>
    /// <returns type="undefined" />
    /// <param name="o[tions" type="json">
    ///     override the defaults in this file
    /// </param>
    $.fn.addGoogleSearch = function (options) {
        return this.each(function () {
            // init UI for blog
            initGSearch($(this));

            var opts = $.extend(true, defaults, options);
            // init autosuggestion
            if (opts.isASOn) {
                initGSearchAutoSuggestion();
            }
        });
    }

    /// <summary>
    ///     init google search UI, which includes the search scopes and the search form
    /// </summary>
    /// <returns type="undefined" />
    /// <param name="$ele" type="jquery object">
    ///     the element the add this google search
    /// </param>
    function initGSearch($ele) {
        var sch = $("<div class='websearch'>\
			<form id='srchfrm' method='get' action='http://www.google.com/search?q=&FORM=&mkt=zh-c'>\
			<div class='scopes'>\
				<a class='first selected' href='http://www.google.com/search?q='>Web</a> <span>|</span><a href='http://www.google.com/images?q='>Images</a> <span>| </span><a href='http://www.google.com/maps?q='>Maps</a> <span>| </span><a href='http://www.google.com/news?q='>News</a> <span>|</span><a class='last' href='http://www.google.com/books?q='>Books</a>\
			</div>\
			<div class='search'>\
				<span class='bo'><span class='bi'>\
					<input type='text' id='q' name='q' autocomplete='off' />\
					<input type='submit' id='sch' value='Google Search' />\
					<input type='hidden' value='FromTestPage' name='form' />\
					<input type='hidden' value='en' name='hl' />\
				</span></span>\
			</div>\
			</form>\
		</div>\
		\
		");

        // append html
        $ele.append(sch);

        var searchFrom = $("#srchfrm");

        // bind scope change event
        $("div.scopes a", searchFrom).click(onScopeChange);

        // bind search button submit event
        // we need do something like update the value attribute for input, or add new input(s)
        searchFrom.bind("submit", onFormSubmit);
    }

    function onScopeChange(event) {
        // prevent default action
        event.preventDefault();

        var link = $(this);  // current element
        var srchForm = link.closest("form");  // the form which event fired

        // remove the "selected" class from original element
        $("a.selected", srchForm).removeClass("selected");

        // add this class to the new element
        link.addClass("selected");

        // store the href value in via $.data()
        srchForm.data("actionUrl", link.attr("href"));

        // focus on input text box
        $("#q", srchForm).focus();
    }

    function onFormSubmit() {
        var srchForm = $(this);

        // get the value in search box
        var searchValue = $("#q", srchForm).val();

        var actionUrl = srchForm.data("actionUrl");

        if (actionUrl) {
            // 设置action url - 这个是未拼接前的
            srchForm.attr("action", srchForm.data("actionUrl"));

            // 解析这个url成为一个name value对的对象
            var queryParams = getParam(actionUrl);

            // 为存在的input设置value或者如果scope的action url里面有但DOM不存在就创建隐藏的input并为name, value赋值
            setFormFields(srchForm, queryParams);
        }
    }

    function setFormFields($searchForm, queryParams) {
        // for each parameter in the query params
        for (var param in queryParams) {
            // 判断queryParams(也就是每个scope的href属性)里面是否有当前的这个属性(比如q, 比如bing的mkt再或者如下面写死的语言)
            // 如果有, 就直接设置这个input的value属性(submit的时候会读取这些信息), 所有这些信息会被拼接到url后面以name=value的形式
            // 如果没有，就在这个form里面创建一个隐藏的空间赋上name和value属性 - 所有这些input的name和value都会被默认全部的拼接到Url后面
            // 所以， 如果你有什么想要的每个搜索都有的参数可以在form里面写死， 比如下面的2个hidden input一个是form另一个是语言
            // 可用于tracking，或者指定搜索语言等。
            if ($("input[name=" + param + "]", $searchForm).attr("value", queryParams[param]).length == 0) {
                $('<input type="hidden"/>').attr(
						{
						    "name": param,
						    "value": queryParams[param]
						}
					).appendTo($searchForm);
            }
        };
    }

    function getParam(url) {
        // 解析每个scope的href url，返回一个所有属性的对象，包含每个的name和value属性
        var params = {};

        if (url) {
            var paramName = (arguments.length > 1) ? $.makeArray(arguments).slice(1).join('|') : '\\w+';
            var regex = new RegExp("\\b(" + paramName + ")=([^&#]+)", "ig");

            var match;

            while ((match = regex.exec(url))) {
                params[match[1][toLowerCase]()] = match[2];
            }
        }
        return params;
    }

    function procressData(response) {

        response = eval(response);

        var asList = $("<ul></ul>");
        var asItem = $("<li></li>");

        for (var i = 0; i < response[1].length; i++) {
            var asItem = $("<li></li>");

            asItem.text(response[1][i][0]);

            asList.append(asItem);
        }

        // show the suggestions
        $("#sw_as").html(asList).css("display", "block");

        // init mouse down
        $("#sw_as li").bind("mousedown", onSelectionMouseDown);

        // init mouse hover
        $("#sw_as li").hover(function () {
            $("#sw_as li").each(function () {
                $(this).removeClass("selected");
            });
            $(this).addClass("selected");
        },
	function () {
	    $(this).removeClass("selected");
	});
    }

    function onSelectionMouseDown(event) {
        delete asMouseDown;

        //add a variable to window if the mouse down happens on AS area
        asMouseDown = event.currentTarget;
    }

    /////////////////////////////////////////////////////////////////////////////////
    // TODO
    // update the attribute of autoComplete
    // Autosuggestion off: autoComplete on
    // Autosuggestion on: autoComplete off
    /////////////////////////////////////////////////////////////////////////////////

    // init the auto suggest when page load
    // we have 3 options(immediately, page loaded, interaction with search input box) to choose in our feature
    // here I will just init it when page load
    function initGSearchAutoSuggestion() {
        $(document).ready(function () {
            var divId = "sw_as";

            var $searchForm = $("#srchfrm");
            var $searchInputBox = $("#q", $searchForm);

            // add the auto suggest div to html
            $autoSuggestContainer = $("<div></div>").attr("id", divId);
            $searchInputBox.closest("div").after($autoSuggestContainer);

            // apply the functionality of AS
            var config = {
                "formId": "srchfrm",
                "searchboxId": "q",
                "containerId": divId,
                "api": "http://google.com.cn/complete/search?",
                "callback": "procressData"
            };
            var suggest = new auto_suggest(config);
            suggest.addAutoSuggest();

            return 1;
        });
    }

    var auto_suggest = function (config) {
        // init the variables according to the config passed in
        var $searchForm = $("#" + config.formId);
        var $searchInputBox = $("#" + config.searchboxId, $searchForm);
        var _apiUrl = config.api;

        var _$currentSuggestion;  //global current suggestion variable
        var _currentQuery;

        // fetch related
        var _dynamicScript;

        // #region Public methods
        // autoSuggest functionality
        // public method for AS
        this.addAutoSuggest = function () {

            // bind events
            $searchForm.bind("submit", onSearch);
            $searchInputBox.bind("keyup", onQueryBoxKeyUp);
            $searchInputBox.bind("keydown", onQueryBoxKeyDown);
            $searchInputBox.bind("click", onQueryBoxClick);
            $searchInputBox.bind("blur", onQueryBoxBlur);
        }

        // #endregion

        // #region AS search events
        function onSearch(event) {
        }

        function onQueryBoxKeyUp(event) {
            var key = event.keyCode;

            //Other than space, return on non-alpha numeric range simply return.
            //Enter key event only occurs if the user uses IME, else the form is submmited without the event occuring.
            if ((key > 8 && key < 38 && key != 32 && key != 13) || (key > 111 && key < 146)) {
                return;
            }

            // 38 and 40 - cursor up and down
            if (key == 40) {
                var _selectedItem = $("#sw_as li.selected");
                if (_selectedItem.html()) {
                    _selectedItem.removeClass("selected");

                    if (_selectedItem.next().html()) {
                        _selectedItem.next().addClass("selected");
                        $searchInputBox.val($("#sw_as li.selected").html());

                        return;
                    }
                }

                $("#sw_as li").first().addClass("selected");
                $searchInputBox.val($("#sw_as li.selected").html());

                return;
            }

            if (key == 38) {
                var _selectedItem = $("#sw_as li.selected");
                if (_selectedItem.html()) {
                    _selectedItem.removeClass("selected");

                    if (_selectedItem.prev().html()) {
                        _selectedItem.prev().addClass("selected");
                        $searchInputBox.val($("#sw_as li.selected").html());

                        return;
                    }
                }

                $("#sw_as li").last().addClass("selected");
                $searchInputBox.val($("#sw_as li.selected").html());

                return;
            }

            // set query string
            setCurrentQuery();

            // ferch google suggestions
            fetchSuggestions();
        }

        function onQueryBoxKeyDown(event) {
        }

        function onQueryBoxClick(event) {
            setCurrentQuery();

            if (_currentQuery)
                fetchSuggestions();
        }

        function onQueryBoxBlur(event) {
            //hide the AS div
            $("#sw_as").css("display", "none");

            //if the click action happens on the AS div
            //submit the form with the selected value
            if (window.asMouseDown) {
                $("#q").val(window.asMouseDown.innerHTML);

                //delete this variable, as we dont need it any more
                //if not, it will impact on next time
                delete window.asMouseDown;

                //submit the form
                $searchForm.submit();
            }
        }

        function fetchSuggestions() {
            // add search key word and call back
            var url = _apiUrl + "q=" + _currentQuery;

            $.ajax({
                type: 'get',
                url: url,
                dataType: 'jsonp',
                success: eval(config.callback)
            });
        }

        function setCurrentQuery() {
            // set current query
            _currentQuery = $.trim($searchInputBox.val());
        }

        // #endregion
    }

})(jQuery);